@@ -8,20 +8,20 @@
 
 #include "core/fpdfapi/parser/cpdf_document.h"
 #include "core/fpdfapi/render/cpdf_dibtransferfunc.h"
-#include "core/fxge/dib/cfx_dibsource.h"
+#include "core/fxge/dib/cfx_dibbase.h"
 
 CPDF_TransferFunc::CPDF_TransferFunc(CPDF_Document* pDoc) : m_pPDFDoc(pDoc) {}
 
 CPDF_TransferFunc::~CPDF_TransferFunc() {}
 
-FX_COLORREF CPDF_TransferFunc::TranslateColor(FX_COLORREF rgb) const {
-  return FXSYS_RGB(m_Samples[FXSYS_GetRValue(rgb)],
-                   m_Samples[256 + FXSYS_GetGValue(rgb)],
-                   m_Samples[512 + FXSYS_GetBValue(rgb)]);
+FX_COLORREF CPDF_TransferFunc::TranslateColor(FX_COLORREF colorref) const {
+  return FXSYS_BGR(m_Samples[512 + FXSYS_GetBValue(colorref)],
+                   m_Samples[256 + FXSYS_GetGValue(colorref)],
+                   m_Samples[FXSYS_GetRValue(colorref)]);
 }
 
-RetainPtr<CFX_DIBSource> CPDF_TransferFunc::TranslateImage(
-    const RetainPtr<CFX_DIBSource>& pSrc) {
+RetainPtr<CFX_DIBBase> CPDF_TransferFunc::TranslateImage(
+    const RetainPtr<CFX_DIBBase>& pSrc) {
   RetainPtr<CPDF_TransferFunc> pHolder(this);
   auto pDest = pdfium::MakeRetain<CPDF_DIBTransferFunc>(pHolder);
   pDest->LoadSrc(pSrc);
