@@ -12,21 +12,26 @@
 #include "core/fxcrt/unowned_ptr.h"
 
 class CPDF_Document;
-class CPDF_Object;
+class CPDF_Array;
 
 class CPDF_Dest {
  public:
   CPDF_Dest();
   CPDF_Dest(const CPDF_Dest& that);
-  explicit CPDF_Dest(CPDF_Object* pObj);
+  explicit CPDF_Dest(const CPDF_Array* pArray);
   ~CPDF_Dest();
 
-  CPDF_Object* GetObject() const { return m_pObj.Get(); }
-  ByteString GetRemoteName();
-  int GetPageIndex(CPDF_Document* pDoc);
-  uint32_t GetPageObjNum();
-  int GetZoomMode();
-  float GetParam(int index);
+  const CPDF_Array* GetArray() const { return m_pArray.Get(); }
+  ByteString GetRemoteName() const;
+
+  int GetDestPageIndex(CPDF_Document* pDoc) const;
+  uint32_t GetPageObjNum() const;
+
+  // Returns the zoom mode, as one of the PDFDEST_VIEW_* values in fpdf_doc.h.
+  int GetZoomMode() const;
+
+  unsigned long GetNumParams() const;
+  float GetParam(int index) const;
 
   bool GetXYZ(bool* pHasX,
               bool* pHasY,
@@ -36,7 +41,7 @@ class CPDF_Dest {
               float* pZoom) const;
 
  private:
-  UnownedPtr<CPDF_Object> m_pObj;
+  UnownedPtr<const CPDF_Array> const m_pArray;
 };
 
 #endif  // CORE_FPDFDOC_CPDF_DEST_H_
