@@ -41,7 +41,7 @@ CPDF_TextPageFind::CPDF_TextPageFind(const CPDF_TextPage* pTextPage)
       m_resStart(0),
       m_resEnd(-1),
       m_IsFind(false) {
-  m_strText = m_pTextPage->GetPageText();
+  m_strText = m_pTextPage->GetAllPageText();
   int nCount = pTextPage->CountChars();
   if (nCount)
     m_CharIndex.push_back(0);
@@ -81,11 +81,11 @@ int CPDF_TextPageFind::GetCharIndex(int index) const {
 
 bool CPDF_TextPageFind::FindFirst(const WideString& findwhat,
                                   int flags,
-                                  pdfium::Optional<size_t> startPos) {
+                                  Optional<size_t> startPos) {
   if (!m_pTextPage)
     return false;
   if (m_strText.IsEmpty() || m_bMatchCase != (flags & FPDFTEXT_MATCHCASE))
-    m_strText = m_pTextPage->GetPageText();
+    m_strText = m_pTextPage->GetAllPageText();
   WideString findwhatStr = findwhat;
   m_findWhat = findwhatStr;
   m_flags = flags;
@@ -142,7 +142,7 @@ bool CPDF_TextPageFind::FindNext() {
     return m_IsFind;
   }
   int nCount = pdfium::CollectionSize<int>(m_csFindWhatArray);
-  pdfium::Optional<size_t> nResultPos = 0;
+  Optional<size_t> nResultPos = 0;
   size_t nStartPos = m_findNextStart.value();
   bool bSpaceStart = false;
   for (int iWord = 0; iWord < nCount; iWord++) {
@@ -161,7 +161,7 @@ bool CPDF_TextPageFind::FindNext() {
       }
       continue;
     }
-    nResultPos = m_strText.Find(csWord.c_str(), nStartPos);
+    nResultPos = m_strText.Find(csWord.AsStringView(), nStartPos);
     if (!nResultPos.has_value()) {
       m_IsFind = false;
       return m_IsFind;
@@ -236,8 +236,7 @@ bool CPDF_TextPageFind::FindPrev() {
     return m_IsFind;
   }
   CPDF_TextPageFind findEngine(m_pTextPage.Get());
-  bool ret =
-      findEngine.FindFirst(m_findWhat, m_flags, pdfium::Optional<size_t>(0));
+  bool ret = findEngine.FindFirst(m_findWhat, m_flags, Optional<size_t>(0));
   if (!ret) {
     m_IsFind = false;
     return m_IsFind;
@@ -279,42 +278,42 @@ void CPDF_TextPageFind::ExtractFindWhat(const WideString& findwhat) {
     return;
   int index = 0;
   while (1) {
-    WideString csWord = TEXT_EMPTY;
-    int ret =
-        ExtractSubString(csWord, findwhat.c_str(), index, TEXT_SPACE_CHAR);
-    if (csWord.IsEmpty()) {
-      if (ret) {
-        m_csFindWhatArray.push_back(L"");
-        index++;
-        continue;
-      } else {
-        break;
-      }
+    Optional<WideString> word =
+        ExtractSubString(findwhat.c_str(), index, TEXT_SPACE_CHAR);
+    if (!word)
+      break;
+
+    if (word->IsEmpty()) {
+      m_csFindWhatArray.push_back(L"");
+      index++;
+      continue;
     }
+
     size_t pos = 0;
-    while (pos < csWord.GetLength()) {
-      WideString curStr = csWord.Mid(pos, 1);
-      wchar_t curChar = csWord[pos];
+    while (pos < word->GetLength()) {
+      WideString curStr = word->Mid(pos, 1);
+      wchar_t curChar = (*word)[pos];
       if (IsIgnoreSpaceCharacter(curChar)) {
         if (pos > 0 && curChar == 0x2019) {
           pos++;
           continue;
         }
         if (pos > 0)
-          m_csFindWhatArray.push_back(csWord.Left(pos));
+          m_csFindWhatArray.push_back(word->Left(pos));
         m_csFindWhatArray.push_back(curStr);
-        if (pos == csWord.GetLength() - 1) {
-          csWord.clear();
+        if (pos == word->GetLength() - 1) {
+          word->clear();
           break;
         }
-        csWord = csWord.Right(csWord.GetLength() - pos - 1);
+        word.emplace(word->Right(word->GetLength() - pos - 1));
         pos = 0;
         continue;
       }
       pos++;
     }
-    if (!csWord.IsEmpty())
-      m_csFindWhatArray.push_back(csWord);
+
+    if (!word->IsEmpty())
+      m_csFindWhatArray.push_back(word.value());
     index++;
   }
 }
@@ -359,29 +358,30 @@ bool CPDF_TextPageFind::IsMatchWholeWord(const WideString& csPageText,
   return true;
 }
 
-bool CPDF_TextPageFind::ExtractSubString(WideString& rString,
-                                         const wchar_t* lpszFullString,
-                                         int iSubString,
-                                         wchar_t chSep) {
+Optional<WideString> CPDF_TextPageFind::ExtractSubString(
+    const wchar_t* lpszFullString,
+    int iSubString,
+    wchar_t chSep) {
   if (!lpszFullString)
-    return false;
+    return {};
+
   while (iSubString--) {
     lpszFullString = std::wcschr(lpszFullString, chSep);
-    if (!lpszFullString) {
-      rString.clear();
-      return false;
-    }
+    if (!lpszFullString)
+      return {};
+
     lpszFullString++;
     while (*lpszFullString == chSep)
       lpszFullString++;
   }
+
   const wchar_t* lpchEnd = std::wcschr(lpszFullString, chSep);
-  int nLen =
-      lpchEnd ? (int)(lpchEnd - lpszFullString) : (int)wcslen(lpszFullString);
-  ASSERT(nLen >= 0);
-  memcpy(rString.GetBuffer(nLen), lpszFullString, nLen * sizeof(wchar_t));
-  rString.ReleaseBuffer(rString.GetStringLength());
-  return true;
+  int nLen = lpchEnd ? static_cast<int>(lpchEnd - lpszFullString)
+                     : static_cast<int>(wcslen(lpszFullString));
+  if (nLen < 0)
+    return {};
+
+  return {WideString(lpszFullString, static_cast<size_t>(nLen))};
 }
 
 int CPDF_TextPageFind::GetCurOrder() const {
