@@ -132,7 +132,7 @@ using CFX_Size = CFX_STemplate<int32_t>;
 using CFX_SizeF = CFX_STemplate<float>;
 
 template <class BaseType>
-class CFX_VTemplate : public CFX_PTemplate<BaseType> {
+class CFX_VTemplate final : public CFX_PTemplate<BaseType> {
  public:
   using CFX_PTemplate<BaseType>::x;
   using CFX_PTemplate<BaseType>::y;
@@ -178,6 +178,7 @@ using CFX_VectorF = CFX_VTemplate<float>;
 // TODO(tsepez): Consolidate all these different rectangle classes.
 
 // LTRB rectangles (y-axis runs downwards).
+// Struct layout is compatible with win32 RECT.
 struct FX_RECT {
   FX_RECT() : left(0), top(0), right(0), bottom(0) {}
   FX_RECT(int l, int t, int r, int b) : left(l), top(t), right(r), bottom(b) {}
@@ -233,14 +234,11 @@ class CFX_FloatRect {
 
   explicit CFX_FloatRect(const FX_RECT& rect);
 
+  static CFX_FloatRect GetBBox(const CFX_PointF* pPoints, int nPoints);
+
   void Normalize();
 
-  void Reset() {
-    left = 0.0f;
-    right = 0.0f;
-    bottom = 0.0f;
-    top = 0.0f;
-  }
+  void Reset();
 
   bool IsEmpty() const { return left >= right || bottom >= top; }
 
@@ -250,12 +248,21 @@ class CFX_FloatRect {
   void Intersect(const CFX_FloatRect& other_rect);
   void Union(const CFX_FloatRect& other_rect);
 
+  // These may be better at rounding than ToFxRect() and friends.
+  //
+  // Returned rect has bounds rounded up/down such that it is contained in the
+  // original.
   FX_RECT GetInnerRect() const;
-  FX_RECT GetOuterRect() const;
-  FX_RECT GetClosestRect() const;
-  CFX_FloatRect GetCenterSquare() const;
 
-  int Substract4(CFX_FloatRect& substract_rect, CFX_FloatRect* pRects);
+  // Returned rect has bounds rounded up/down such that the original is
+  // contained in it.
+  FX_RECT GetOuterRect() const;
+
+  // Returned rect has bounds rounded up/down such that the dimensions are
+  // rounded up and the sum of the error in the bounds is minimized.
+  FX_RECT GetClosestRect() const;
+
+  CFX_FloatRect GetCenterSquare() const;
 
   void InitRect(const CFX_PointF& point) {
     left = point.x;
@@ -331,25 +338,19 @@ class CFX_FloatRect {
     bottom += f;
   }
 
-  void Scale(float fScale) {
-    float fHalfWidth = (right - left) / 2.0f;
-    float fHalfHeight = (top - bottom) / 2.0f;
+  void Scale(float fScale);
+  void ScaleFromCenterPoint(float fScale);
 
-    float center_x = (left + right) / 2;
-    float center_y = (top + bottom) / 2;
+  // GetInnerRect() and friends may be better at rounding than these methods.
+  // Unlike the methods above, these two blindly floor / round the LBRT values.
+  // Doing so may introduce rounding errors that are visible to users as
+  // off-by-one pixels/lines.
+  //
+  // Floors LBRT values.
+  FX_RECT ToFxRect() const;
 
-    left = center_x - fHalfWidth * fScale;
-    bottom = center_y - fHalfHeight * fScale;
-    right = center_x + fHalfWidth * fScale;
-    top = center_y + fHalfHeight * fScale;
-  }
-
-  static CFX_FloatRect GetBBox(const CFX_PointF* pPoints, int nPoints);
-
-  FX_RECT ToFxRect() const {
-    return FX_RECT(static_cast<int32_t>(left), static_cast<int32_t>(top),
-                   static_cast<int32_t>(right), static_cast<int32_t>(bottom));
-  }
+  // Rounds LBRT values.
+  FX_RECT ToRoundedFxRect() const;
 
   float left;
   float bottom;
@@ -357,80 +358,54 @@ class CFX_FloatRect {
   float top;
 };
 
-// LTWH rectangles (y-axis runs downwards).
-template <class BaseType>
-class CFX_RTemplate {
- public:
-  using PointType = CFX_PTemplate<BaseType>;
-  using SizeType = CFX_STemplate<BaseType>;
-  using VectorType = CFX_VTemplate<BaseType>;
-  using RectType = CFX_RTemplate<BaseType>;
+#ifndef NDEBUG
+std::ostream& operator<<(std::ostream& os, const CFX_FloatRect& rect);
+#endif
 
-  CFX_RTemplate() : left(0), top(0), width(0), height(0) {}
-  CFX_RTemplate(BaseType dst_left,
-                BaseType dst_top,
-                BaseType dst_width,
-                BaseType dst_height)
+// LTWH rectangles (y-axis runs downwards).
+class CFX_RectF {
+ public:
+  using PointType = CFX_PointF;
+  using SizeType = CFX_SizeF;
+
+  CFX_RectF() : left(0), top(0), width(0), height(0) {}
+  CFX_RectF(float dst_left, float dst_top, float dst_width, float dst_height)
       : left(dst_left), top(dst_top), width(dst_width), height(dst_height) {}
-  CFX_RTemplate(BaseType dst_left, BaseType dst_top, const SizeType& dst_size)
+  CFX_RectF(float dst_left, float dst_top, const SizeType& dst_size)
       : left(dst_left),
         top(dst_top),
         width(dst_size.width),
         height(dst_size.height) {}
-  CFX_RTemplate(const PointType& p, BaseType dst_width, BaseType dst_height)
+  CFX_RectF(const PointType& p, float dst_width, float dst_height)
       : left(p.x), top(p.y), width(dst_width), height(dst_height) {}
-  CFX_RTemplate(const PointType& p1, const SizeType& s2)
+  CFX_RectF(const PointType& p1, const SizeType& s2)
       : left(p1.x), top(p1.y), width(s2.width), height(s2.height) {}
-  CFX_RTemplate(const PointType& p1, const PointType& p2)
-      : left(p1.x),
-        top(p1.y),
-        width(p2.width - p1.width),
-        height(p2.height - p1.height) {
-    Normalize();
-  }
-  CFX_RTemplate(const PointType& p, const VectorType& v)
-      : left(p.x), top(p.y), width(v.x), height(v.y) {
-    Normalize();
-  }
-
-  explicit CFX_RTemplate(const CFX_FloatRect& r)
-      : left(static_cast<BaseType>(r.left)),
-        top(static_cast<BaseType>(r.top)),
-        width(static_cast<BaseType>(r.Width())),
-        height(static_cast<BaseType>(r.Height())) {}
 
   // NOLINTNEXTLINE(runtime/explicit)
-  CFX_RTemplate(const RectType& other)
+  CFX_RectF(const CFX_RectF& other)
       : left(other.left),
         top(other.top),
         width(other.width),
         height(other.height) {}
 
-  template <typename OtherType>
-  CFX_RTemplate<OtherType> As() const {
-    return CFX_RTemplate<OtherType>(
-        static_cast<OtherType>(left), static_cast<OtherType>(top),
-        static_cast<OtherType>(width), static_cast<OtherType>(height));
-  }
-
   void Reset() {
     left = 0;
     top = 0;
     width = 0;
     height = 0;
   }
-  RectType& operator+=(const PointType& p) {
+  CFX_RectF& operator+=(const PointType& p) {
     left += p.x;
     top += p.y;
     return *this;
   }
-  RectType& operator-=(const PointType& p) {
+  CFX_RectF& operator-=(const PointType& p) {
     left -= p.x;
     top -= p.y;
     return *this;
   }
-  BaseType right() const { return left + width; }
-  BaseType bottom() const { return top + height; }
+  float right() const { return left + width; }
+  float bottom() const { return top + height; }
   void Normalize() {
     if (width < 0) {
       left += width;
@@ -441,46 +416,46 @@ class CFX_RTemplate {
       height = -height;
     }
   }
-  void Offset(BaseType dx, BaseType dy) {
+  void Offset(float dx, float dy) {
     left += dx;
     top += dy;
   }
-  void Inflate(BaseType x, BaseType y) {
+  void Inflate(float x, float y) {
     left -= x;
     width += x * 2;
     top -= y;
     height += y * 2;
   }
   void Inflate(const PointType& p) { Inflate(p.x, p.y); }
-  void Inflate(BaseType off_left,
-               BaseType off_top,
-               BaseType off_right,
-               BaseType off_bottom) {
+  void Inflate(float off_left,
+               float off_top,
+               float off_right,
+               float off_bottom) {
     left -= off_left;
     top -= off_top;
     width += off_left + off_right;
     height += off_top + off_bottom;
   }
-  void Inflate(const RectType& rt) {
+  void Inflate(const CFX_RectF& rt) {
     Inflate(rt.left, rt.top, rt.left + rt.width, rt.top + rt.height);
   }
-  void Deflate(BaseType x, BaseType y) {
+  void Deflate(float x, float y) {
     left += x;
     width -= x * 2;
     top += y;
     height -= y * 2;
   }
   void Deflate(const PointType& p) { Deflate(p.x, p.y); }
-  void Deflate(BaseType off_left,
-               BaseType off_top,
-               BaseType off_right,
-               BaseType off_bottom) {
+  void Deflate(float off_left,
+               float off_top,
+               float off_right,
+               float off_bottom) {
     left += off_left;
     top += off_top;
     width -= off_left + off_right;
     height -= off_top + off_bottom;
   }
-  void Deflate(const RectType& rt) {
+  void Deflate(const CFX_RectF& rt) {
     Deflate(rt.left, rt.top, rt.top + rt.width, rt.top + rt.height);
   }
   bool IsEmpty() const { return width <= 0 || height <= 0; }
@@ -492,12 +467,12 @@ class CFX_RTemplate {
     return p.x >= left && p.x < left + width && p.y >= top &&
            p.y < top + height;
   }
-  bool Contains(const RectType& rt) const {
+  bool Contains(const CFX_RectF& rt) const {
     return rt.left >= left && rt.right() <= right() && rt.top >= top &&
            rt.bottom() <= bottom();
   }
-  BaseType Width() const { return width; }
-  BaseType Height() const { return height; }
+  float Width() const { return width; }
+  float Height() const { return height; }
   SizeType Size() const { return SizeType(width, height); }
   PointType TopLeft() const { return PointType(left, top); }
   PointType TopRight() const { return PointType(left + width, top); }
@@ -508,9 +483,9 @@ class CFX_RTemplate {
   PointType Center() const {
     return PointType(left + width / 2, top + height / 2);
   }
-  void Union(BaseType x, BaseType y) {
-    BaseType r = right();
-    BaseType b = bottom();
+  void Union(float x, float y) {
+    float r = right();
+    float b = bottom();
 
     left = std::min(left, x);
     top = std::min(top, y);
@@ -521,9 +496,9 @@ class CFX_RTemplate {
     height = b - top;
   }
   void Union(const PointType& p) { Union(p.x, p.y); }
-  void Union(const RectType& rt) {
-    BaseType r = right();
-    BaseType b = bottom();
+  void Union(const CFX_RectF& rt) {
+    float r = right();
+    float b = bottom();
 
     left = std::min(left, rt.left);
     top = std::min(top, rt.top);
@@ -533,9 +508,9 @@ class CFX_RTemplate {
     width = r - left;
     height = b - top;
   }
-  void Intersect(const RectType& rt) {
-    BaseType r = right();
-    BaseType b = bottom();
+  void Intersect(const CFX_RectF& rt) {
+    float r = right();
+    float b = bottom();
 
     left = std::max(left, rt.left);
     top = std::max(top, rt.top);
@@ -545,21 +520,21 @@ class CFX_RTemplate {
     width = r - left;
     height = b - top;
   }
-  bool IntersectWith(const RectType& rt) const {
-    RectType rect = rt;
+  bool IntersectWith(const CFX_RectF& rt) const {
+    CFX_RectF rect = rt;
     rect.Intersect(*this);
     return !rect.IsEmpty();
   }
-  bool IntersectWith(const RectType& rt, float fEpsilon) const {
-    RectType rect = rt;
+  bool IntersectWith(const CFX_RectF& rt, float fEpsilon) const {
+    CFX_RectF rect = rt;
     rect.Intersect(*this);
     return !rect.IsEmpty(fEpsilon);
   }
-  friend bool operator==(const RectType& rc1, const RectType& rc2) {
+  friend bool operator==(const CFX_RectF& rc1, const CFX_RectF& rc2) {
     return rc1.left == rc2.left && rc1.top == rc2.top &&
            rc1.width == rc2.width && rc1.height == rc2.height;
   }
-  friend bool operator!=(const RectType& rc1, const RectType& rc2) {
+  friend bool operator!=(const CFX_RectF& rc1, const CFX_RectF& rc2) {
     return !(rc1 == rc2);
   }
 
@@ -569,13 +544,15 @@ class CFX_RTemplate {
     return CFX_FloatRect(left, top, right(), bottom());
   }
 
-  BaseType left;
-  BaseType top;
-  BaseType width;
-  BaseType height;
+  float left;
+  float top;
+  float width;
+  float height;
 };
-using CFX_Rect = CFX_RTemplate<int32_t>;
-using CFX_RectF = CFX_RTemplate<float>;
+
+#ifndef NDEBUG
+std::ostream& operator<<(std::ostream& os, const CFX_RectF& rect);
+#endif  // NDEBUG
 
 // The matrix is of the form:
 // | a  b  0 |
@@ -590,13 +567,7 @@ class CFX_Matrix {
   explicit CFX_Matrix(const float n[6])
       : a(n[0]), b(n[1]), c(n[2]), d(n[3]), e(n[4]), f(n[5]) {}
 
-  CFX_Matrix(const CFX_Matrix& other)
-      : a(other.a),
-        b(other.b),
-        c(other.c),
-        d(other.d),
-        e(other.e),
-        f(other.f) {}
+  CFX_Matrix(const CFX_Matrix& other) = default;
 
   CFX_Matrix(float a1, float b1, float c1, float d1, float e1, float f1)
       : a(a1), b(b1), c(c1), d(d1), e(e1), f(f1) {}
@@ -621,8 +592,10 @@ class CFX_Matrix {
 
   CFX_Matrix GetInverse() const;
 
-  void Concat(const CFX_Matrix& m, bool bPrepended = false);
-  void ConcatInverse(const CFX_Matrix& m, bool bPrepended = false);
+  void Concat(const CFX_Matrix& right);
+  void ConcatPrepend(const CFX_Matrix& left);
+  void ConcatInverse(const CFX_Matrix& m);
+  void ConcatInversePrepend(const CFX_Matrix& m);
 
   bool IsIdentity() const {
     return a == 1 && b == 0 && c == 0 && d == 1 && e == 0 && f == 0;
@@ -632,16 +605,18 @@ class CFX_Matrix {
   bool IsScaled() const;
   bool WillScale() const { return a != 1.0f || b != 0 || c != 0 || d != 1.0f; }
 
-  void Translate(float x, float y, bool bPrepended = false);
-  void Translate(int32_t x, int32_t y, bool bPrepended = false) {
-    Translate(static_cast<float>(x), static_cast<float>(y), bPrepended);
+  void Translate(float x, float y);
+  void TranslatePrepend(float x, float y);
+  void Translate(int32_t x, int32_t y) {
+    Translate(static_cast<float>(x), static_cast<float>(y));
+  }
+  void TranslatePrepend(int32_t x, int32_t y) {
+    TranslatePrepend(static_cast<float>(x), static_cast<float>(y));
   }
 
-  void Scale(float sx, float sy, bool bPrepended = false);
-  void Rotate(float fRadian, bool bPrepended = false);
-  void RotateAt(float fRadian, float x, float y, bool bPrepended = false);
-
-  void Shear(float fAlphaRadian, float fBetaRadian, bool bPrepended = false);
+  void Scale(float sx, float sy);
+  void Rotate(float fRadian);
+  void Shear(float fAlphaRadian, float fBetaRadian);
 
   void MatchRect(const CFX_FloatRect& dest, const CFX_FloatRect& src);
 
@@ -650,7 +625,6 @@ class CFX_Matrix {
   CFX_FloatRect GetUnitRect() const;
 
   float TransformXDistance(float dx) const;
-  float TransformDistance(float dx, float dy) const;
   float TransformDistance(float distance) const;
 
   CFX_PointF Transform(const CFX_PointF& point) const;
@@ -669,9 +643,6 @@ class CFX_Matrix {
   float d;
   float e;
   float f;
-
- private:
-  void ConcatInternal(const CFX_Matrix& other, bool prepend);
 };
 
 #endif  // CORE_FXCRT_FX_COORDINATES_H_
