@@ -13,15 +13,15 @@
 #include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/retain_ptr.h"
 #include "core/fxge/dib/cfx_dibitmap.h"
-#include "core/fxge/dib/ifx_scanlinecomposer.h"
+#include "core/fxge/dib/scanlinecomposer_iface.h"
 #include "third_party/base/stl_util.h"
 
-class CFX_BitmapStorer : public IFX_ScanlineComposer {
+class CFX_BitmapStorer final : public ScanlineComposerIface {
  public:
   CFX_BitmapStorer();
   ~CFX_BitmapStorer() override;
 
-  // IFX_ScanlineComposer
+  // ScanlineComposerIface
   void ComposeScanline(int line,
                        const uint8_t* scanline,
                        const uint8_t* scan_extra_alpha) override;
