@@ -92,7 +92,7 @@ class AutoClosedCommand {
   ByteString close_;
 };
 
-class AutoClosedQCommand : public AutoClosedCommand {
+class AutoClosedQCommand final : public AutoClosedCommand {
  public:
   explicit AutoClosedQCommand(std::ostringstream* stream)
       : AutoClosedCommand(stream, kStateSaveOperator, kStateRestoreOperator) {}
@@ -104,19 +104,19 @@ ByteString GetColorAppStream(const CFX_Color& color,
   std::ostringstream sColorStream;
 
   switch (color.nColorType) {
-    case COLORTYPE_RGB:
+    case CFX_Color::kRGB:
       sColorStream << color.fColor1 << " " << color.fColor2 << " "
                    << color.fColor3 << " "
                    << (bFillOrStroke ? kSetRGBOperator : kSetRGBStrokedOperator)
                    << "\n";
       break;
-    case COLORTYPE_GRAY:
+    case CFX_Color::kGray:
       sColorStream << color.fColor1 << " "
                    << (bFillOrStroke ? kSetGrayOperator
                                      : kSetGrayStrokedOperator)
                    << "\n";
       break;
-    case COLORTYPE_CMYK:
+    case CFX_Color::kCMYK:
       sColorStream << color.fColor1 << " " << color.fColor2 << " "
                    << color.fColor3 << " " << color.fColor4 << " "
                    << (bFillOrStroke ? kSetCMYKOperator
@@ -129,8 +129,8 @@ ByteString GetColorAppStream(const CFX_Color& color,
 }
 
 ByteString GetAP_Check(const CFX_FloatRect& crBBox) {
-  const float fWidth = crBBox.right - crBBox.left;
-  const float fHeight = crBBox.top - crBBox.bottom;
+  const float fWidth = crBBox.Width();
+  const float fHeight = crBBox.Height();
 
   CFX_PointF pts[8][3] = {{CFX_PointF(0.28f, 0.52f), CFX_PointF(0.27f, 0.48f),
                            CFX_PointF(0.29f, 0.40f)},
@@ -180,8 +180,8 @@ ByteString GetAP_Check(const CFX_FloatRect& crBBox) {
 ByteString GetAP_Circle(const CFX_FloatRect& crBBox) {
   std::ostringstream csAP;
 
-  float fWidth = crBBox.right - crBBox.left;
-  float fHeight = crBBox.top - crBBox.bottom;
+  float fWidth = crBBox.Width();
+  float fHeight = crBBox.Height();
 
   CFX_PointF pt1(crBBox.left, crBBox.bottom + fHeight / 2);
   CFX_PointF pt2(crBBox.left + fWidth / 2, crBBox.top);
@@ -236,8 +236,8 @@ ByteString GetAP_Cross(const CFX_FloatRect& crBBox) {
 ByteString GetAP_Diamond(const CFX_FloatRect& crBBox) {
   std::ostringstream csAP;
 
-  float fWidth = crBBox.right - crBBox.left;
-  float fHeight = crBBox.top - crBBox.bottom;
+  float fWidth = crBBox.Width();
+  float fHeight = crBBox.Height();
 
   CFX_PointF pt1(crBBox.left, crBBox.bottom + fHeight / 2);
   CFX_PointF pt2(crBBox.left + fWidth / 2, crBBox.top);
@@ -298,8 +298,8 @@ ByteString GetAP_Star(const CFX_FloatRect& crBBox) {
 ByteString GetAP_HalfCircle(const CFX_FloatRect& crBBox, float fRotate) {
   std::ostringstream csAP;
 
-  float fWidth = crBBox.right - crBBox.left;
-  float fHeight = crBBox.top - crBBox.bottom;
+  float fWidth = crBBox.Width();
+  float fHeight = crBBox.Height();
 
   CFX_PointF pt1(-fWidth / 2, 0);
   CFX_PointF pt2(0, fHeight / 2);
@@ -514,18 +514,18 @@ ByteString GetCheckBoxAppStream(const CFX_FloatRect& rcBBox,
     case CheckStyle::kCheck:
       return GetAppStream_Check(rcCenter, crText);
     case CheckStyle::kCircle:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Circle(rcCenter, crText);
     case CheckStyle::kCross:
       return GetAppStream_Cross(rcCenter, crText);
     case CheckStyle::kDiamond:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Diamond(rcCenter, crText);
     case CheckStyle::kSquare:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Square(rcCenter, crText);
     case CheckStyle::kStar:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Star(rcCenter, crText);
   }
 }
@@ -539,18 +539,18 @@ ByteString GetRadioButtonAppStream(const CFX_FloatRect& rcBBox,
     case CheckStyle::kCheck:
       return GetAppStream_Check(rcCenter, crText);
     case CheckStyle::kCircle:
-      rcCenter.Scale(1.0f / 2.0f);
+      rcCenter.ScaleFromCenterPoint(1.0f / 2.0f);
       return GetAppStream_Circle(rcCenter, crText);
     case CheckStyle::kCross:
       return GetAppStream_Cross(rcCenter, crText);
     case CheckStyle::kDiamond:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Diamond(rcCenter, crText);
     case CheckStyle::kSquare:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Square(rcCenter, crText);
     case CheckStyle::kStar:
-      rcCenter.Scale(2.0f / 3.0f);
+      rcCenter.ScaleFromCenterPoint(2.0f / 3.0f);
       return GetAppStream_Star(rcCenter, crText);
   }
 }
@@ -776,7 +776,7 @@ ByteString GetPushButtonAppStream(const CFX_FloatRect& rcBBox,
     case ButtonStyle::kIconTopLabelBottom:
       if (pIconStream) {
         if (IsFloatZero(fFontSize)) {
-          fHeight = rcBBox.top - rcBBox.bottom;
+          fHeight = rcBBox.Height();
           rcLabel = CFX_FloatRect(rcBBox.left, rcBBox.bottom, rcBBox.right,
                                   rcBBox.bottom + fHeight * fAutoFontScale);
           rcIcon =
@@ -800,7 +800,7 @@ ByteString GetPushButtonAppStream(const CFX_FloatRect& rcBBox,
     case ButtonStyle::kIconBottomLabelTop:
       if (pIconStream) {
         if (IsFloatZero(fFontSize)) {
-          fHeight = rcBBox.top - rcBBox.bottom;
+          fHeight = rcBBox.Height();
           rcLabel =
               CFX_FloatRect(rcBBox.left, rcBBox.top - fHeight * fAutoFontScale,
                             rcBBox.right, rcBBox.top);
@@ -1055,7 +1055,7 @@ ByteString GetDropButtonAppStream(const CFX_FloatRect& rcBBox) {
   std::ostringstream sAppStream;
   {
     AutoClosedQCommand q(&sAppStream);
-    sAppStream << GetColorAppStream(CFX_Color(COLORTYPE_RGB, 220.0f / 255.0f,
+    sAppStream << GetColorAppStream(CFX_Color(CFX_Color::kRGB, 220.0f / 255.0f,
                                               220.0f / 255.0f, 220.0f / 255.0f),
                                     true)
                << rcBBox.left << " " << rcBBox.bottom << " "
@@ -1067,9 +1067,9 @@ ByteString GetDropButtonAppStream(const CFX_FloatRect& rcBBox) {
   {
     AutoClosedQCommand q(&sAppStream);
     sAppStream << GetBorderAppStreamInternal(
-        rcBBox, 2, CFX_Color(COLORTYPE_GRAY, 0), CFX_Color(COLORTYPE_GRAY, 1),
-        CFX_Color(COLORTYPE_GRAY, 0.5), BorderStyle::BEVELED,
-        CPWL_Dash(3, 0, 0));
+        rcBBox, 2, CFX_Color(CFX_Color::kGray, 0),
+        CFX_Color(CFX_Color::kGray, 1), CFX_Color(CFX_Color::kGray, 0.5),
+        BorderStyle::BEVELED, CPWL_Dash(3, 0, 0));
   }
 
   CFX_PointF ptCenter = CFX_PointF((rcBBox.left + rcBBox.right) / 2,
@@ -1164,29 +1164,34 @@ void CPWL_AppStream::SetAsPushButton() {
       break;
     case BorderStyle::BEVELED:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 1);
       crRightBottom = crBackground / 2.0f;
       break;
     case BorderStyle::INSET:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0.5);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 0.75);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0.5);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 0.75);
       break;
     default:
       break;
   }
 
   CFX_FloatRect rcClient = rcWindow.GetDeflated(fBorderWidth, fBorderWidth);
-  CFX_Color crText(COLORTYPE_GRAY, 0);
+  CFX_Color crText(CFX_Color::kGray, 0);
   ByteString csNameTag;
   CPDF_DefaultAppearance da = pControl->GetDefaultAppearance();
-  if (da.HasColor()) {
-    da.GetColor(iColorType, fc);
+  Optional<CFX_Color::Type> color = da.GetColor(fc);
+  if (color) {
+    iColorType = *color;
     crText = CFX_Color(iColorType, fc[0], fc[1], fc[2], fc[3]);
   }
-  float fFontSize = 12.0f;
-  if (da.HasFont())
-    csNameTag = da.GetFont(&fFontSize);
+
+  float fFontSize;
+  Optional<ByteString> font = da.GetFont(&fFontSize);
+  if (font)
+    csNameTag = *font;
+  else
+    fFontSize = 12.0f;
 
   WideString csWCaption;
   WideString csNormalCaption;
@@ -1287,8 +1292,8 @@ void CPWL_AppStream::SetAsPushButton() {
         break;
       }
       case BorderStyle::INSET: {
-        crLeftTop = CFX_Color(COLORTYPE_GRAY, 0);
-        crRightBottom = CFX_Color(COLORTYPE_GRAY, 1);
+        crLeftTop = CFX_Color(CFX_Color::kGray, 0);
+        crRightBottom = CFX_Color(CFX_Color::kGray, 1);
         break;
       }
       default:
@@ -1339,13 +1344,13 @@ void CPWL_AppStream::SetAsCheckBox() {
       break;
     case BorderStyle::BEVELED:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 1);
       crRightBottom = crBackground / 2.0f;
       break;
     case BorderStyle::INSET:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0.5);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 0.75);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0.5);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 0.75);
       break;
     default:
       break;
@@ -1354,8 +1359,9 @@ void CPWL_AppStream::SetAsCheckBox() {
   CFX_FloatRect rcWindow = widget_->GetRotatedRect();
   CFX_FloatRect rcClient = rcWindow.GetDeflated(fBorderWidth, fBorderWidth);
   CPDF_DefaultAppearance da = pControl->GetDefaultAppearance();
-  if (da.HasColor()) {
-    da.GetColor(iColorType, fc);
+  Optional<CFX_Color::Type> color = da.GetColor(fc);
+  if (color) {
+    iColorType = *color;
     crText = CFX_Color(iColorType, fc[0], fc[1], fc[2], fc[3]);
   }
 
@@ -1399,8 +1405,8 @@ void CPWL_AppStream::SetAsCheckBox() {
       break;
     }
     case BorderStyle::INSET: {
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 1);
       break;
     }
     default:
@@ -1455,13 +1461,13 @@ void CPWL_AppStream::SetAsRadioButton() {
       break;
     case BorderStyle::BEVELED:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 1);
       crRightBottom = crBackground / 2.0f;
       break;
     case BorderStyle::INSET:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0.5);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 0.75);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0.5);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 0.75);
       break;
     default:
       break;
@@ -1470,8 +1476,9 @@ void CPWL_AppStream::SetAsRadioButton() {
   CFX_FloatRect rcWindow = widget_->GetRotatedRect();
   CFX_FloatRect rcClient = rcWindow.GetDeflated(fBorderWidth, fBorderWidth);
   CPDF_DefaultAppearance da = pControl->GetDefaultAppearance();
-  if (da.HasColor()) {
-    da.GetColor(iColorType, fc);
+  Optional<CFX_Color::Type> color = da.GetColor(fc);
+  if (color) {
+    iColorType = *color;
     crText = CFX_Color(iColorType, fc[0], fc[1], fc[2], fc[3]);
   }
 
@@ -1504,11 +1511,11 @@ void CPWL_AppStream::SetAsRadioButton() {
   CFX_FloatRect rcCenter = rcWindow.GetCenterSquare().GetDeflated(1.0f, 1.0f);
   if (nStyle == CheckStyle::kCircle) {
     if (nBorderStyle == BorderStyle::BEVELED) {
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 1);
       crRightBottom = crBackground - 0.25f;
     } else if (nBorderStyle == BorderStyle::INSET) {
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0.5f);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 0.75f);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0.5f);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 0.75f);
     }
 
     csAP_N_ON =
@@ -1532,8 +1539,8 @@ void CPWL_AppStream::SetAsRadioButton() {
       break;
     }
     case BorderStyle::INSET: {
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 1);
       break;
     }
     default:
@@ -1546,11 +1553,11 @@ void CPWL_AppStream::SetAsRadioButton() {
     CFX_Color crBK = crBackground - 0.25f;
     if (nBorderStyle == BorderStyle::BEVELED) {
       crLeftTop = crBackground - 0.25f;
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 1);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 1);
       crBK = crBackground;
     } else if (nBorderStyle == BorderStyle::INSET) {
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 1);
     }
 
     csAP_D_ON =
@@ -1590,12 +1597,13 @@ void CPWL_AppStream::SetAsComboBox(const WideString* sValue) {
   rcButton.left = rcButton.right - 13;
   rcButton.Normalize();
 
-  auto pEdit = pdfium::MakeUnique<CPWL_EditImpl>();
-  pEdit->EnableRefresh(false);
-
+  // Font map must outlive |pEdit|.
   CBA_FontMap font_map(
       widget_.Get(),
       widget_->GetInterForm()->GetFormFillEnv()->GetSysHandler());
+
+  auto pEdit = pdfium::MakeUnique<CPWL_EditImpl>();
+  pEdit->EnableRefresh(false);
   pEdit->SetFontMap(&font_map);
 
   CFX_FloatRect rcEdit = rcClient;
@@ -1656,12 +1664,13 @@ void CPWL_AppStream::SetAsListBox() {
   CFX_FloatRect rcClient = widget_->GetClientRect();
   std::ostringstream sBody;
 
-  auto pEdit = pdfium::MakeUnique<CPWL_EditImpl>();
-  pEdit->EnableRefresh(false);
-
+  // Font map must outlive |pEdit|.
   CBA_FontMap font_map(
       widget_.Get(),
       widget_->GetInterForm()->GetFormFillEnv()->GetSysHandler());
+
+  auto pEdit = pdfium::MakeUnique<CPWL_EditImpl>();
+  pEdit->EnableRefresh(false);
   pEdit->SetFontMap(&font_map);
   pEdit->SetPlateRect(CFX_FloatRect(rcClient.left, 0.0f, rcClient.right, 0.0f));
 
@@ -1695,7 +1704,7 @@ void CPWL_AppStream::SetAsListBox() {
           CFX_FloatRect(rcClient.left, fy - fItemHeight, rcClient.right, fy);
       {
         AutoClosedQCommand q(&sList);
-        sList << GetColorAppStream(CFX_Color(COLORTYPE_RGB, 0, 51.0f / 255.0f,
+        sList << GetColorAppStream(CFX_Color(CFX_Color::kRGB, 0, 51.0f / 255.0f,
                                              113.0f / 255.0f),
                                    true)
               << rcItem.left << " " << rcItem.bottom << " " << rcItem.Width()
@@ -1704,7 +1713,7 @@ void CPWL_AppStream::SetAsListBox() {
       }
 
       AutoClosedCommand bt(&sList, kTextBeginOperator, kTextEndOperator);
-      sList << GetColorAppStream(CFX_Color(COLORTYPE_GRAY, 1), true)
+      sList << GetColorAppStream(CFX_Color(CFX_Color::kGray, 1), true)
             << GetEditAppStream(pEdit.get(), CFX_PointF(0.0f, fy), true, 0);
     } else {
       CFX_Color crText = widget_->GetTextPWLColor();
@@ -1740,12 +1749,13 @@ void CPWL_AppStream::SetAsTextField(const WideString* sValue) {
   std::ostringstream sBody;
   std::ostringstream sLines;
 
-  auto pEdit = pdfium::MakeUnique<CPWL_EditImpl>();
-  pEdit->EnableRefresh(false);
-
+  // Font map must outlive |pEdit|.
   CBA_FontMap font_map(
       widget_.Get(),
       widget_->GetInterForm()->GetFormFillEnv()->GetSysHandler());
+
+  auto pEdit = pdfium::MakeUnique<CPWL_EditImpl>();
+  pEdit->EnableRefresh(false);
   pEdit->SetFontMap(&font_map);
 
   CFX_FloatRect rcClient = widget_->GetClientRect();
@@ -1774,7 +1784,7 @@ void CPWL_AppStream::SetAsTextField(const WideString* sValue) {
 #ifdef PDF_ENABLE_XFA
   WideString sValueTmp;
   if (!sValue && widget_->GetMixXFAWidget()) {
-    sValueTmp = widget_->GetValue(true);
+    sValueTmp = widget_->GetValue();
     sValue = &sValueTmp;
   }
 #endif  // PDF_ENABLE_XFA
@@ -1903,9 +1913,8 @@ void CPWL_AppStream::AddImage(const ByteString& sAPType, CPDF_Stream* pImage) {
 
   CPDF_Dictionary* pXObject =
       pStreamResList->SetNewFor<CPDF_Dictionary>("XObject");
-  pXObject->SetNewFor<CPDF_Reference>(sImageAlias,
-                                      widget_->GetPageView()->GetPDFDocument(),
-                                      pImage->GetObjNum());
+  pXObject->SetFor(sImageAlias, pImage->MakeReference(
+                                    widget_->GetPageView()->GetPDFDocument()));
 }
 
 void CPWL_AppStream::Write(const ByteString& sAPType,
@@ -1928,7 +1937,7 @@ void CPWL_AppStream::Write(const ByteString& sAPType,
   if (!pStream) {
     CPDF_Document* doc = widget_->GetPageView()->GetPDFDocument();
     pStream = doc->NewIndirect<CPDF_Stream>();
-    pParentDict->SetNewFor<CPDF_Reference>(sAPType, doc, pStream->GetObjNum());
+    pParentDict->SetFor(sAPType, pStream->MakeReference(doc));
   }
 
   CPDF_Dictionary* pStreamDict = pStream->GetDict();
@@ -1939,12 +1948,11 @@ void CPWL_AppStream::Write(const ByteString& sAPType,
     pStreamDict->SetNewFor<CPDF_Name>("Type", "XObject");
     pStreamDict->SetNewFor<CPDF_Name>("Subtype", "Form");
     pStreamDict->SetNewFor<CPDF_Number>("FormType", 1);
-    pStream->InitStream(nullptr, 0, std::move(pNewDict));
+    pStream->InitStream({}, std::move(pNewDict));
   }
   pStreamDict->SetMatrixFor("Matrix", widget_->GetMatrix());
   pStreamDict->SetRectFor("BBox", widget_->GetRotatedRect());
-  pStream->SetDataAndRemoveFilter((uint8_t*)(sContents.c_str()),
-                                  sContents.GetLength());
+  pStream->SetDataAndRemoveFilter(sContents.AsRawSpan());
 }
 
 void CPWL_AppStream::Remove(const ByteString& sAPType) {
@@ -1953,7 +1961,7 @@ void CPWL_AppStream::Remove(const ByteString& sAPType) {
 
 ByteString CPWL_AppStream::GetBackgroundAppStream() const {
   CFX_Color crBackground = widget_->GetFillPWLColor();
-  if (crBackground.nColorType != COLORTYPE_TRANSPARENT)
+  if (crBackground.nColorType != CFX_Color::kTransparent)
     return GetRectFillAppStream(widget_->GetRotatedRect(), crBackground);
 
   return ByteString();
@@ -1976,13 +1984,13 @@ ByteString CPWL_AppStream::GetBorderAppStream() const {
       break;
     case BorderStyle::BEVELED:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 1);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 1);
       crRightBottom = crBackground / 2.0f;
       break;
     case BorderStyle::INSET:
       fBorderWidth *= 2;
-      crLeftTop = CFX_Color(COLORTYPE_GRAY, 0.5);
-      crRightBottom = CFX_Color(COLORTYPE_GRAY, 0.75);
+      crLeftTop = CFX_Color(CFX_Color::kGray, 0.5);
+      crRightBottom = CFX_Color(CFX_Color::kGray, 0.75);
       break;
     default:
       break;
