@@ -23,13 +23,8 @@ constexpr float kDefaultFontSize = 9.0f;
 
 CPWL_Wnd::CreateParams::CreateParams()
     : rcRectWnd(0, 0, 0, 0),
-      pSystemHandler(nullptr),
-      pFontMap(nullptr),
-      pProvider(nullptr),
-      pFocusHandler(nullptr),
       dwFlags(0),
       sBackgroundColor(),
-      pAttachedWidget(nullptr),
       nBorderStyle(BorderStyle::SOLID),
       dwBorderWidth(1),
       sBorderColor(),
@@ -37,8 +32,6 @@ CPWL_Wnd::CreateParams::CreateParams()
       nTransparency(255),
       fFontSize(kDefaultFontSize),
       sDash(3, 0, 0),
-      pAttachedData(nullptr),
-      pParentWnd(nullptr),
       pMsgControl(nullptr),
       eCursorType(FXCT_ARROW) {}
 
@@ -46,32 +39,15 @@ CPWL_Wnd::CreateParams::CreateParams(const CreateParams& other) = default;
 
 CPWL_Wnd::CreateParams::~CreateParams() = default;
 
-class CPWL_MsgControl : public Observable<CPWL_MsgControl> {
-  friend class CPWL_Wnd;
-
+class CPWL_MsgControl final : public Observable<CPWL_MsgControl> {
  public:
-  explicit CPWL_MsgControl(CPWL_Wnd* pWnd) {
-    m_pCreatedWnd = pWnd;
-    Default();
-  }
-
-  ~CPWL_MsgControl() { Default(); }
-
-  void Default() {
-    m_aMousePath.clear();
-    m_aKeyboardPath.clear();
-    m_pMainMouseWnd = nullptr;
-    m_pMainKeyboardWnd = nullptr;
-  }
+  explicit CPWL_MsgControl(CPWL_Wnd* pWnd) : m_pCreatedWnd(pWnd) {}
+  ~CPWL_MsgControl() {}
 
   bool IsWndCreated(const CPWL_Wnd* pWnd) const {
     return m_pCreatedWnd == pWnd;
   }
 
-  bool IsMainCaptureMouse(const CPWL_Wnd* pWnd) const {
-    return pWnd == m_pMainMouseWnd;
-  }
-
   bool IsWndCaptureMouse(const CPWL_Wnd* pWnd) const {
     return pWnd && pdfium::ContainsValue(m_aMousePath, pWnd);
   }
@@ -114,7 +90,6 @@ class CPWL_MsgControl : public Observable<CPWL_MsgControl> {
   void SetCapture(CPWL_Wnd* pWnd) {
     m_aMousePath.clear();
     if (pWnd) {
-      m_pMainMouseWnd = pWnd;
       CPWL_Wnd* pParent = pWnd;
       while (pParent) {
         m_aMousePath.push_back(pParent);
@@ -124,15 +99,15 @@ class CPWL_MsgControl : public Observable<CPWL_MsgControl> {
   }
 
   void ReleaseCapture() {
-    m_pMainMouseWnd = nullptr;
     m_aMousePath.clear();
   }
 
+  CPWL_Wnd* GetFocusedWindow() const { return m_pMainKeyboardWnd.Get(); }
+
  private:
   std::vector<CPWL_Wnd*> m_aMousePath;
   std::vector<CPWL_Wnd*> m_aKeyboardPath;
   UnownedPtr<CPWL_Wnd> m_pCreatedWnd;
-  UnownedPtr<CPWL_Wnd> m_pMainMouseWnd;
   UnownedPtr<CPWL_Wnd> m_pMainKeyboardWnd;
 };
 
@@ -295,33 +270,32 @@ void CPWL_Wnd::DrawChildAppearance(CFX_RenderDevice* pDevice,
 }
 
 bool CPWL_Wnd::InvalidateRect(CFX_FloatRect* pRect) {
-  ObservedPtr thisObserved(this);
-  if (!IsValid())
+    if (!IsValid())
     return true;
 
+  ObservedPtr thisObserved(this);
   CFX_FloatRect rcRefresh = pRect ? *pRect : GetWindowRect();
-
   if (!HasFlag(PWS_NOREFRESHCLIP)) {
     CFX_FloatRect rcClip = GetClipRect();
-    if (!rcClip.IsEmpty()) {
+    if (!rcClip.IsEmpty())
       rcRefresh.Intersect(rcClip);
-    }
   }
 
   CFX_FloatRect rcWin = PWLtoWnd(rcRefresh);
   rcWin.Inflate(1, 1);
   rcWin.Normalize();
 
-  if (CFX_SystemHandler* pSH = GetSystemHandler()) {
-    if (CPDFSDK_Widget* widget = static_cast<CPDFSDK_Widget*>(
-            m_CreationParams.pAttachedWidget.Get())) {
-      pSH->InvalidateRect(widget, rcWin);
-      if (!thisObserved)
-        return false;
-    }
-  }
+  CFX_SystemHandler* pSH = GetSystemHandler();
+  if (!pSH)
+    return true;
 
-  return true;
+  CPDFSDK_Widget* widget =
+      ToCPDFSDKWidget(m_CreationParams.pAttachedWidget.Get());
+  if (!widget)
+    return true;
+
+  pSH->InvalidateRect(widget, rcWin);
+  return !!thisObserved;
 }
 
 #define PWL_IMPLEMENT_KEY_METHOD(key_method_name)                  \
@@ -373,12 +347,32 @@ PWL_IMPLEMENT_MOUSE_METHOD(OnRButtonUp)
 PWL_IMPLEMENT_MOUSE_METHOD(OnMouseMove)
 #undef PWL_IMPLEMENT_MOUSE_METHOD
 
+WideString CPWL_Wnd::GetText() {
+  return WideString();
+}
+
 WideString CPWL_Wnd::GetSelectedText() {
   return WideString();
 }
 
 void CPWL_Wnd::ReplaceSelection(const WideString& text) {}
 
+bool CPWL_Wnd::CanUndo() {
+  return false;
+}
+
+bool CPWL_Wnd::CanRedo() {
+  return false;
+}
+
+bool CPWL_Wnd::Undo() {
+  return false;
+}
+
+bool CPWL_Wnd::Redo() {
+  return false;
+}
+
 bool CPWL_Wnd::OnMouseWheel(short zDelta,
                             const CFX_PointF& point,
                             uint32_t nFlag) {
@@ -421,10 +415,6 @@ void CPWL_Wnd::NotifyLButtonUp(CPWL_Wnd* child, const CFX_PointF& pos) {}
 
 void CPWL_Wnd::NotifyMouseMove(CPWL_Wnd* child, const CFX_PointF& pos) {}
 
-CPWL_Wnd* CPWL_Wnd::GetParentWindow() const {
-  return m_CreationParams.pParentWnd;
-}
-
 CFX_FloatRect CPWL_Wnd::GetWindowRect() const {
   return m_rcWindow;
 }
@@ -496,10 +486,6 @@ const CPWL_Dash& CPWL_Wnd::GetBorderDash() const {
   return m_CreationParams.sDash;
 }
 
-CPWL_Wnd::PrivateData* CPWL_Wnd::GetAttachedData() const {
-  return m_CreationParams.pAttachedData.Get();
-}
-
 CPWL_ScrollBar* CPWL_Wnd::GetVScrollBar() const {
   return HasFlag(PWS_VSCROLL) ? m_pVScrollBar.Get() : nullptr;
 }
@@ -568,11 +554,6 @@ bool CPWL_Wnd::ClientHitTest(const CFX_PointF& point) const {
   return IsValid() && IsVisible() && GetClientRect().Contains(point);
 }
 
-const CPWL_Wnd* CPWL_Wnd::GetRootWnd() const {
-  auto* pParent = m_CreationParams.pParentWnd;
-  return pParent ? pParent->GetRootWnd() : this;
-}
-
 bool CPWL_Wnd::SetVisible(bool bVisible) {
   if (!IsValid())
     return true;
@@ -698,27 +679,15 @@ void CPWL_Wnd::SetFontSize(float fFontSize) {
 }
 
 CFX_SystemHandler* CPWL_Wnd::GetSystemHandler() const {
-  return m_CreationParams.pSystemHandler;
-}
-
-CPWL_Wnd::FocusHandlerIface* CPWL_Wnd::GetFocusHandler() const {
-  return m_CreationParams.pFocusHandler.Get();
-}
-
-CPWL_Wnd::ProviderIface* CPWL_Wnd::GetProvider() const {
-  return m_CreationParams.pProvider.Get();
-}
-
-IPVT_FontMap* CPWL_Wnd::GetFontMap() const {
-  return m_CreationParams.pFontMap;
+  return m_CreationParams.pSystemHandler.Get();
 }
 
 CFX_Color CPWL_Wnd::GetBorderLeftTopColor(BorderStyle nBorderStyle) const {
   switch (nBorderStyle) {
     case BorderStyle::BEVELED:
-      return CFX_Color(COLORTYPE_GRAY, 1);
+      return CFX_Color(CFX_Color::kGray, 1);
     case BorderStyle::INSET:
-      return CFX_Color(COLORTYPE_GRAY, 0.5f);
+      return CFX_Color(CFX_Color::kGray, 0.5f);
     default:
       return CFX_Color();
   }
@@ -729,7 +698,7 @@ CFX_Color CPWL_Wnd::GetBorderRightBottomColor(BorderStyle nBorderStyle) const {
     case BorderStyle::BEVELED:
       return GetBackgroundColor() / 2.0f;
     case BorderStyle::INSET:
-      return CFX_Color(COLORTYPE_GRAY, 0.75f);
+      return CFX_Color(CFX_Color::kGray, 0.75f);
     default:
       return CFX_Color();
   }
@@ -804,7 +773,7 @@ void CPWL_Wnd::SetChildMatrix(const CFX_Matrix& mt) {
 
 const CPWL_Wnd* CPWL_Wnd::GetFocused() const {
   CPWL_MsgControl* pMsgCtrl = GetMsgControl();
-  return pMsgCtrl ? pMsgCtrl->m_pMainKeyboardWnd.Get() : nullptr;
+  return pMsgCtrl ? pMsgCtrl->GetFocusedWindow() : nullptr;
 }
 
 void CPWL_Wnd::EnableWindow(bool bEnable) {
