@@ -9,25 +9,22 @@
 
 #include <vector>
 
-#include "core/fxcrt/fx_basic.h"
+#include "core/fxcrt/fx_string.h"
 
 class CBC_UtilCodingConvert {
  public:
   CBC_UtilCodingConvert();
   virtual ~CBC_UtilCodingConvert();
-  static void UnicodeToLocale(const CFX_WideString& source,
-                              CFX_ByteString& result);
-  static void LocaleToUtf8(const CFX_ByteString& source,
-                           CFX_ByteString& result);
-  static void LocaleToUtf8(const CFX_ByteString& source,
+  static void UnicodeToLocale(const WideString& source, ByteString& result);
+  static void LocaleToUtf8(const ByteString& source, ByteString& result);
+  static void LocaleToUtf8(const ByteString& source,
                            std::vector<uint8_t>& result);
   static void Utf8ToLocale(const std::vector<uint8_t>& source,
-                           CFX_ByteString& result);
+                           ByteString& result);
   static void Utf8ToLocale(const uint8_t* source,
                            int32_t count,
-                           CFX_ByteString& result);
-  static void UnicodeToUTF8(const CFX_WideString& source,
-                            CFX_ByteString& result);
+                           ByteString& result);
+  static void UnicodeToUTF8(const WideString& source, ByteString& result);
 };
 
 #endif  // FXBARCODE_BC_UTILCODINGCONVERT_H_
