@@ -399,48 +399,49 @@ CBC_BarcodeMatrix* CBC_PDF417::getBarcodeMatrix() {
   return m_barcodeMatrix.get();
 }
 
-void CBC_PDF417::generateBarcodeLogic(CFX_WideString msg,
-                                      int32_t errorCorrectionLevel,
-                                      int32_t& e) {
+bool CBC_PDF417::generateBarcodeLogic(WideString msg,
+                                      int32_t errorCorrectionLevel) {
   int32_t errorCorrectionCodeWords =
       CBC_PDF417ErrorCorrection::getErrorCorrectionCodewordCount(
-          errorCorrectionLevel, e);
-  if (e != BCExceptionNO)
-    return;
-  CFX_WideString highLevel =
+          errorCorrectionLevel);
+  if (errorCorrectionCodeWords < 0)
+    return false;
+
+  int32_t e = BCExceptionNO;
+  WideString highLevel =
       CBC_PDF417HighLevelEncoder::encodeHighLevel(msg, m_compaction, e);
   if (e != BCExceptionNO)
-    return;
+    return false;
   int32_t sourceCodeWords = highLevel.GetLength();
-  std::vector<int32_t>* dimension =
-      determineDimensions(sourceCodeWords, errorCorrectionCodeWords, e);
-  if (e != BCExceptionNO)
-    return;
-  int32_t cols = (*dimension)[0];
-  int32_t rows = (*dimension)[1];
-  delete dimension;
+  std::vector<int32_t> dimensions =
+      determineDimensions(sourceCodeWords, errorCorrectionCodeWords);
+  if (dimensions.size() != 2)
+    return false;
+  int32_t cols = dimensions[0];
+  int32_t rows = dimensions[1];
   int32_t pad = getNumberOfPadCodewords(sourceCodeWords,
                                         errorCorrectionCodeWords, cols, rows);
-  if (sourceCodeWords + errorCorrectionCodeWords + 1 > 929) {
-    e = BCExceptionEncodedMessageContainsTooManyCodeWords;
-    return;
-  }
+  if (sourceCodeWords + errorCorrectionCodeWords + 1 > 929)
+    return false;
+
   int32_t n = sourceCodeWords + pad + 1;
-  CFX_WideString sb;
+  WideString sb;
   sb += (wchar_t)n;
   sb += highLevel;
-  for (int32_t i = 0; i < pad; i++) {
+  for (int32_t i = 0; i < pad; i++)
     sb += (wchar_t)900;
+
+  WideString dataCodewords(sb);
+  WideString ec;
+  if (!CBC_PDF417ErrorCorrection::generateErrorCorrection(
+          dataCodewords, errorCorrectionLevel, &ec)) {
+    return false;
   }
-  CFX_WideString dataCodewords(sb);
-  CFX_WideString ec = CBC_PDF417ErrorCorrection::generateErrorCorrection(
-      dataCodewords, errorCorrectionLevel, e);
-  if (e != BCExceptionNO)
-    return;
-  CFX_WideString fullCodewords = dataCodewords + ec;
-  m_barcodeMatrix = pdfium::MakeUnique<CBC_BarcodeMatrix>(rows, cols);
+  WideString fullCodewords = dataCodewords + ec;
+  m_barcodeMatrix = pdfium::MakeUnique<CBC_BarcodeMatrix>(cols, rows);
   encodeLowLevel(fullCodewords, cols, rows, errorCorrectionLevel,
                  m_barcodeMatrix.get());
+  return true;
 }
 
 void CBC_PDF417::setDimensions(int32_t maxCols,
@@ -497,16 +498,16 @@ void CBC_PDF417::encodeChar(int32_t pattern,
   logic->addBar(last, width);
 }
 
-void CBC_PDF417::encodeLowLevel(CFX_WideString fullCodewords,
+void CBC_PDF417::encodeLowLevel(WideString fullCodewords,
                                 int32_t c,
                                 int32_t r,
                                 int32_t errorCorrectionLevel,
                                 CBC_BarcodeMatrix* logic) {
   int32_t idx = 0;
   for (int32_t y = 0; y < r; y++) {
+    CBC_BarcodeRow* logicRow = logic->getRow(y);
     int32_t cluster = y % 3;
-    logic->startRow();
-    encodeChar(START_PATTERN, 17, logic->getCurrentRow());
+    encodeChar(START_PATTERN, 17, logicRow);
     int32_t left;
     int32_t right;
     if (cluster == 0) {
@@ -520,65 +521,57 @@ void CBC_PDF417::encodeLowLevel(CFX_WideString fullCodewords,
       right = (30 * (y / 3)) + (errorCorrectionLevel * 3) + ((r - 1) % 3);
     }
     int32_t pattern = CODEWORD_TABLE[cluster][left];
-    encodeChar(pattern, 17, logic->getCurrentRow());
+    encodeChar(pattern, 17, logicRow);
     for (int32_t x = 0; x < c; x++) {
-      pattern = CODEWORD_TABLE[cluster][fullCodewords.GetAt(idx)];
-      encodeChar(pattern, 17, logic->getCurrentRow());
+      pattern = CODEWORD_TABLE[cluster][fullCodewords[idx]];
+      encodeChar(pattern, 17, logicRow);
       idx++;
     }
     if (m_compact) {
-      encodeChar(STOP_PATTERN, 1, logic->getCurrentRow());
+      encodeChar(STOP_PATTERN, 1, logicRow);
     } else {
       pattern = CODEWORD_TABLE[cluster][right];
-      encodeChar(pattern, 17, logic->getCurrentRow());
-      encodeChar(STOP_PATTERN, 18, logic->getCurrentRow());
+      encodeChar(pattern, 17, logicRow);
+      encodeChar(STOP_PATTERN, 18, logicRow);
     }
   }
 }
 
-std::vector<int32_t>* CBC_PDF417::determineDimensions(
+std::vector<int32_t> CBC_PDF417::determineDimensions(
     int32_t sourceCodeWords,
-    int32_t errorCorrectionCodeWords,
-    int32_t& e) {
+    int32_t errorCorrectionCodeWords) const {
+  std::vector<int32_t> dimensions;
   float ratio = 0.0f;
-  std::vector<int32_t>* dimension = nullptr;
   for (int32_t cols = m_minCols; cols <= m_maxCols; cols++) {
     int32_t rows =
         calculateNumberOfRows(sourceCodeWords, errorCorrectionCodeWords, cols);
-    if (rows < m_minRows) {
+    if (rows < m_minRows)
       break;
-    }
-    if (rows > m_maxRows) {
+    if (rows > m_maxRows)
       continue;
-    }
     float newRatio =
         ((17 * cols + 69) * DEFAULT_MODULE_WIDTH) / (rows * HEIGHT);
-    if (dimension &&
+    if (!dimensions.empty() &&
         fabsf(newRatio - PREFERRED_RATIO) > fabsf(ratio - PREFERRED_RATIO)) {
       continue;
     }
     ratio = newRatio;
-    delete dimension;
-    dimension = new std::vector<int32_t>;
-    dimension->push_back(cols);
-    dimension->push_back(rows);
+    dimensions.resize(2);
+    dimensions[0] = cols;
+    dimensions[1] = rows;
   }
-  if (!dimension) {
+  if (dimensions.empty()) {
     int32_t rows = calculateNumberOfRows(sourceCodeWords,
-                                         errorCorrectionCodeWords, m_minCols);
+                                         errorCorrectionCodeWords, m_maxCols);
     if (rows < m_minRows) {
-      dimension = new std::vector<int32_t>;
-      dimension->push_back(m_minCols);
-      dimension->push_back(m_minRows);
+      dimensions.resize(2);
+      dimensions[0] = m_maxCols;
+      dimensions[1] = m_minRows;
     } else if (rows >= 3 && rows <= 90) {
-      dimension = new std::vector<int32_t>;
-      dimension->push_back(m_minCols);
-      dimension->push_back(rows);
+      dimensions.resize(2);
+      dimensions[0] = m_maxCols;
+      dimensions[1] = rows;
     }
   }
-  if (!dimension) {
-    e = BCExceptionUnableToFitMessageInColumns;
-    return nullptr;
-  }
-  return dimension;
+  return dimensions;
 }
