@@ -7,14 +7,14 @@
 #ifndef FXJS_CJS_DISPLAY_H_
 #define FXJS_CJS_DISPLAY_H_
 
-#include "fxjs/JS_Define.h"
+#include "fxjs/js_define.h"
 
-class CJS_Display : public CJS_Object {
+class CJS_Display final : public CJS_Object {
  public:
   static void DefineJSObjects(CFXJS_Engine* pEngine);
 
-  explicit CJS_Display(v8::Local<v8::Object> pObject) : CJS_Object(pObject) {}
-  ~CJS_Display() override {}
+  CJS_Display(v8::Local<v8::Object> pObject, CJS_Runtime* pRuntime);
+  ~CJS_Display() override;
 
  private:
   static int ObjDefnID;
