@@ -7,22 +7,22 @@
 #ifndef FXJS_XFA_CJX_FORM_H_
 #define FXJS_XFA_CJX_FORM_H_
 
-#include "fxjs/CJX_Define.h"
+#include "fxjs/jse_define.h"
 #include "fxjs/xfa/cjx_model.h"
 
 class CXFA_Form;
 
-class CJX_Form : public CJX_Model {
+class CJX_Form final : public CJX_Model {
  public:
   explicit CJX_Form(CXFA_Form* form);
   ~CJX_Form() override;
 
-  JS_METHOD(execCalculate, CJX_Form);
-  JS_METHOD(execInitialize, CJX_Form);
-  JS_METHOD(execValidate, CJX_Form);
-  JS_METHOD(formNodes, CJX_Form);
-  JS_METHOD(recalculate, CJX_Form);
-  JS_METHOD(remerge, CJX_Form);
+  JSE_METHOD(execCalculate, CJX_Form);
+  JSE_METHOD(execInitialize, CJX_Form);
+  JSE_METHOD(execValidate, CJX_Form);
+  JSE_METHOD(formNodes, CJX_Form);
+  JSE_METHOD(recalculate, CJX_Form);
+  JSE_METHOD(remerge, CJX_Form);
 
  private:
   static const CJX_MethodSpec MethodSpecs[];
