@@ -7,10 +7,12 @@
 // Copyright (C) 2011-2015, 2017 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2012 Arseny Solokha <asolokha@gmx.com>
 // Copyright (C) 2012 Fabio D'Urso <fabiodurso@hotmail.it>
-// Copyright (C) 2012, 2014, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2012, 2014, 2017, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2013 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2015 Arthur Stavisky <vovodroid@gmail.com>
+// Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+// Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 //
 //========================================================================
 
@@ -21,8 +23,8 @@
 #include <poppler-config.h>
 #include <vector>
 
-static GBool printVersion = gFalse;
-static GBool printHelp = gFalse;
+static bool printVersion = false;
+static bool printHelp = false;
 
 static const ArgDesc argDesc[] = {
   {"-v", argFlag, &printVersion, 0,
@@ -35,10 +37,10 @@ static const ArgDesc argDesc[] = {
    "print usage information"},
   {"-?", argFlag, &printHelp, 0,
    "print usage information"},
-  {NULL}
+  { }
 };
 
-void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum, int newRefNum, Dict *srcNameTree, Dict *mergeNameTree, int numOffset) {
+static void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum, int newRefNum, Dict *srcNameTree, Dict *mergeNameTree, int numOffset) {
   Object mergeNameArray = mergeNameTree->lookup("Names");
   Object srcNameArray = srcNameTree->lookup("Names");
   if (mergeNameArray.isArray() && srcNameArray.isArray()) {
@@ -53,7 +55,7 @@ void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum,
           Object mvalue = mergeNameArray.arrayGetNF(j + 1);
           if (mkey.isString() && mvalue.isRef()) {
             if (mkey.getString()->cmp(key.getString()) < 0) {
-              newNameArray->add(Object(new GooString(mkey.getString()->getCString())));
+              newNameArray->add(Object(new GooString(mkey.getString()->c_str())));
               newNameArray->add(Object(mvalue.getRef().num + numOffset, mvalue.getRef().gen));
               j += 2;
             } else if (mkey.getString()->cmp(key.getString()) == 0) {
@@ -65,7 +67,7 @@ void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum,
             j += 2;
           }
         }
-        newNameArray->add(Object(new GooString(key.getString()->getCString())));
+        newNameArray->add(Object(new GooString(key.getString()->c_str())));
         newNameArray->add(Object(value.getRef().num, value.getRef().gen));
       }
     }
@@ -73,7 +75,7 @@ void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum,
       Object mkey = mergeNameArray.arrayGetNF(j);
       Object mvalue = mergeNameArray.arrayGetNF(j + 1);
       if (mkey.isString() && mvalue.isRef()) {
-        newNameArray->add(Object(new GooString(mkey.getString()->getCString())));
+        newNameArray->add(Object(new GooString(mkey.getString()->c_str())));
         newNameArray->add(Object(mvalue.getRef().num + numOffset, mvalue.getRef().gen));
       }
       j += 2;
@@ -86,16 +88,16 @@ void doMergeNameTree(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum,
       Object key = mergeNameArray.arrayGetNF(i);
       Object value = mergeNameArray.arrayGetNF(i + 1);
       if (key.isString() && value.isRef()) {
-        newNameArray->add(Object(new GooString(key.getString()->getCString())));
+        newNameArray->add(Object(new GooString(key.getString()->c_str())));
         newNameArray->add(Object(value.getRef().num + numOffset, value.getRef().gen));
       }
     }
-    srcNameTree->add(copyString("Names"), Object(newNameArray));
+    srcNameTree->add("Names", Object(newNameArray));
     doc->markPageObjects(mergeNameTree, srcXRef, countRef, numOffset, oldRefNum, newRefNum);
   }
 }
 
-void doMergeNameDict(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum, int newRefNum, Dict *srcNameDict, Dict *mergeNameDict, int numOffset) {
+static void doMergeNameDict(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum, int newRefNum, Dict *srcNameDict, Dict *mergeNameDict, int numOffset) {
   for (int i = 0; i < mergeNameDict->getLength(); i++) {
     const char *key = mergeNameDict->getKey(i);
     Object mergeNameTree = mergeNameDict->lookup(key);
@@ -105,12 +107,12 @@ void doMergeNameDict(PDFDoc *doc, XRef *srcXRef, XRef *countRef, int oldRefNum,
     } else if (srcNameTree.isNull() && mergeNameTree.isDict()) {
       Object newNameTree(new Dict(srcXRef));
       doMergeNameTree(doc, srcXRef, countRef, oldRefNum, newRefNum, newNameTree.getDict(), mergeNameTree.getDict(), numOffset);
-      srcNameDict->add(copyString(key), std::move(newNameTree));
+      srcNameDict->add(key, std::move(newNameTree));
     }
   }
 }
 
-void doMergeFormDict(Dict *srcFormDict, Dict *mergeFormDict, int numOffset) {
+static void doMergeFormDict(Dict *srcFormDict, Dict *mergeFormDict, int numOffset) {
   Object srcFields = srcFormDict->lookup("Fields");
   Object mergeFields = mergeFormDict->lookup("Fields");
   if (srcFields.isArray() && mergeFields.isArray()) {
@@ -129,9 +131,9 @@ int main (int argc, char *argv[])
 ///////////////////////////////////////////////////////////////////////////
 {
   int objectsCount = 0;
-  Guint numOffset = 0;
+  unsigned int numOffset = 0;
   std::vector<Object> pages;
-  std::vector<Guint> offsets;
+  std::vector<unsigned int> offsets;
   XRef *yRef, *countRef;
   FILE *f;
   OutStream *outStr;
@@ -144,7 +146,7 @@ int main (int argc, char *argv[])
   int exitCode;
 
   exitCode = 99;
-  const GBool ok = parseArgs (argDesc, &argc, argv);
+  const bool ok = parseArgs (argDesc, &argc, argv);
   if (!ok || argc < 3 || printVersion || printHelp) {
     fprintf(stderr, "pdfunite version %s\n", PACKAGE_VERSION);
     fprintf(stderr, "%s\n", popplerCopyright);
@@ -162,7 +164,7 @@ int main (int argc, char *argv[])
 
   for (i = 1; i < argc - 1; i++) {
     GooString *gfileName = new GooString(argv[i]);
-    PDFDoc *doc = new PDFDoc(gfileName, NULL, NULL, NULL);
+    PDFDoc *doc = new PDFDoc(gfileName, nullptr, nullptr, nullptr);
     if (doc->isOk() && !doc->isEncrypted()) {
       docs.push_back(doc);
       if (doc->getPDFMajorVersion() > majorVersion) {
@@ -190,7 +192,7 @@ int main (int argc, char *argv[])
 
   yRef = new XRef();
   countRef = new XRef();
-  yRef->add(0, 65535, 0, gFalse);
+  yRef->add(0, 65535, 0, false);
   PDFDoc::writeHeader(outStr, majorVersion, minorVersion);
 
   // handle OutputIntents, AcroForm, OCProperties & Names
@@ -226,16 +228,16 @@ int main (int argc, char *argv[])
             if (intent.isDict()) {
               Object idf = intent.dictLookup("OutputConditionIdentifier");
               if (idf.isString()) {
-                GooString *gidf = idf.getString();
-                GBool removeIntent = gTrue;
+                const GooString *gidf = idf.getString();
+                bool removeIntent = true;
                 for (int k = 0; k < pageintents.arrayGetLength(); k++) {
                   Object pgintent = pageintents.arrayGet(k, 0);
                   if (pgintent.isDict()) {
                     Object pgidf = pgintent.dictLookup("OutputConditionIdentifier");
                     if (pgidf.isString()) {
-                      GooString *gpgidf = pgidf.getString();
+                      const GooString *gpgidf = pgidf.getString();
                       if (gpgidf->cmp(gidf) == 0) {
-                        removeIntent = gFalse;
+                        removeIntent = false;
                         break;
                       }
                     }
@@ -244,7 +246,7 @@ int main (int argc, char *argv[])
                 if (removeIntent) {
                   intents.arrayRemove(j);
                   error(errSyntaxWarning, -1, "Output intent {0:s} missing in pdf {1:s}, removed",
-                   gidf->getCString(), docs[i]->getFileName()->getCString());
+                   gidf->c_str(), docs[i]->getFileName()->c_str());
                 }
               } else {
                 intents.arrayRemove(j);
@@ -278,7 +280,7 @@ int main (int argc, char *argv[])
         continue;
       }
 
-      PDFRectangle *cropBox = NULL;
+      const PDFRectangle *cropBox = nullptr;
       if (docs[i]->getCatalog()->getPage(j)->isCropped())
         cropBox = docs[i]->getCatalog()->getPage(j)->getCropBox();
       docs[i]->replacePageDict(j,
@@ -316,12 +318,12 @@ int main (int argc, char *argv[])
         doMergeFormDict(afObj.getDict(), pageForm.getDict(), numOffset);
       }
     }
-    objectsCount += docs[i]->writePageObjects(outStr, yRef, numOffset, gTrue);
+    objectsCount += docs[i]->writePageObjects(outStr, yRef, numOffset, true);
     numOffset = yRef->getNumObjects() + 1;
   }
 
   rootNum = yRef->getNumObjects() + 1;
-  yRef->add(rootNum, 0, outStr->getPos(), gTrue);
+  yRef->add(rootNum, 0, outStr->getPos(), true);
   outStr->printf("%d 0 obj\n", rootNum);
   outStr->printf("<< /Type /Catalog /Pages %d 0 R", rootNum + 1);
   // insert OutputIntents
@@ -330,7 +332,7 @@ int main (int argc, char *argv[])
     for (j = 0; j < intents.arrayGetLength(); j++) {
       Object intent = intents.arrayGet(j, 0);
       if (intent.isDict()) {
-        PDFDoc::writeObject(&intent, outStr, yRef, 0, NULL, cryptRC4, 0, 0, 0);
+        PDFDoc::writeObject(&intent, outStr, yRef, 0, nullptr, cryptRC4, 0, 0, 0);
       }
     }
     outStr->printf("]");
@@ -338,22 +340,22 @@ int main (int argc, char *argv[])
   // insert AcroForm
   if (!afObj.isNull()) {
     outStr->printf(" /AcroForm ");
-    PDFDoc::writeObject(&afObj, outStr, yRef, 0, NULL, cryptRC4, 0, 0, 0);
+    PDFDoc::writeObject(&afObj, outStr, yRef, 0, nullptr, cryptRC4, 0, 0, 0);
   }
   // insert OCProperties
   if (!ocObj.isNull() && ocObj.isDict()) {
     outStr->printf(" /OCProperties ");
-    PDFDoc::writeObject(&ocObj, outStr, yRef, 0, NULL, cryptRC4, 0, 0, 0);
+    PDFDoc::writeObject(&ocObj, outStr, yRef, 0, nullptr, cryptRC4, 0, 0, 0);
   }
   // insert Names
   if (!names.isNull() && names.isDict()) {
     outStr->printf(" /Names ");
-    PDFDoc::writeObject(&names, outStr, yRef, 0, NULL, cryptRC4, 0, 0, 0);
+    PDFDoc::writeObject(&names, outStr, yRef, 0, nullptr, cryptRC4, 0, 0, 0);
   }
   outStr->printf(">>\nendobj\n");
   objectsCount++;
 
-  yRef->add(rootNum + 1, 0, outStr->getPos(), gTrue);
+  yRef->add(rootNum + 1, 0, outStr->getPos(), true);
   outStr->printf("%d 0 obj\n", rootNum + 1);
   outStr->printf("<< /Type /Pages /Kids [");
   for (j = 0; j < (int) pages.size(); j++)
@@ -362,7 +364,7 @@ int main (int argc, char *argv[])
   objectsCount++;
 
   for (i = 0; i < (int) pages.size(); i++) {
-    yRef->add(rootNum + i + 2, 0, outStr->getPos(), gTrue);
+    yRef->add(rootNum + i + 2, 0, outStr->getPos(), true);
     outStr->printf("%d 0 obj\n", rootNum + i + 2);
     outStr->printf("<< ");
     Dict *pageDict = pages[i].getDict();
@@ -375,7 +377,7 @@ int main (int argc, char *argv[])
         outStr->printf("/Parent %d 0 R", rootNum + 1);
       } else {
         outStr->printf("/%s ", key);
-        PDFDoc::writeObject(&value, outStr, yRef, offsets[i], NULL, cryptRC4, 0, 0, 0);
+        PDFDoc::writeObject(&value, outStr, yRef, offsets[i], nullptr, cryptRC4, 0, 0, 0);
       }
     }
     outStr->printf(" >>\nendobj\n");
@@ -385,9 +387,9 @@ int main (int argc, char *argv[])
   Ref ref;
   ref.num = rootNum;
   ref.gen = 0;
-  Object trailerDict = PDFDoc::createTrailerDict(objectsCount, gFalse, 0, &ref, yRef,
+  Object trailerDict = PDFDoc::createTrailerDict(objectsCount, false, 0, &ref, yRef,
                                                 fileName, outStr->getPos());
-  PDFDoc::writeXRefTableTrailer(std::move(trailerDict), yRef, gTrue, // write all entries according to ISO 32000-1, 7.5.4 Cross-Reference Table: "For a file that has never been incrementally updated, the cross-reference section shall contain only one subsection, whose object numbering begins at 0."
+  PDFDoc::writeXRefTableTrailer(std::move(trailerDict), yRef, true, // write all entries according to ISO 32000-1, 7.5.4 Cross-Reference Table: "For a file that has never been incrementally updated, the cross-reference section shall contain only one subsection, whose object numbering begins at 0."
                                 uxrefOffset, outStr, yRef);
 
   outStr->close();
