@@ -21,7 +21,7 @@ CXFA_EventParam::CXFA_EventParam()
       m_bShift(false),
       m_bIsFormReady(false) {}
 
-CXFA_EventParam::~CXFA_EventParam() {}
+CXFA_EventParam::~CXFA_EventParam() = default;
 
 CXFA_EventParam::CXFA_EventParam(const CXFA_EventParam& other) = default;
 
@@ -33,7 +33,6 @@ void CXFA_EventParam::Reset() {
   m_bKeyDown = false;
   m_bModifier = false;
   m_wsNewContentType.clear();
-  m_wsNewText.clear();
   m_wsPrevContentType.clear();
   m_wsPrevText.clear();
   m_bReenter = false;
@@ -44,3 +43,8 @@ void CXFA_EventParam::Reset() {
   m_wsSoapFaultString.clear();
   m_bIsFormReady = false;
 }
+
+WideString CXFA_EventParam::GetNewText() const {
+  return m_wsPrevText.Left(m_iSelStart) + m_wsChange +
+         m_wsPrevText.Right(m_wsPrevText.GetLength() - m_iSelEnd);
+}
