@@ -7,12 +7,16 @@
 #ifndef XFA_FXFA_CXFA_FFCHECKBUTTON_H_
 #define XFA_FXFA_CXFA_FFCHECKBUTTON_H_
 
+#include "core/fxcrt/unowned_ptr.h"
 #include "xfa/fxfa/cxfa_fffield.h"
 #include "xfa/fxfa/cxfa_ffpageview.h"
+#include "xfa/fxfa/parser/cxfa_node.h"
 
-class CXFA_FFCheckButton : public CXFA_FFField {
+class CXFA_CheckButton;
+
+class CXFA_FFCheckButton final : public CXFA_FFField {
  public:
-  explicit CXFA_FFCheckButton(CXFA_WidgetAcc* pDataAcc);
+  CXFA_FFCheckButton(CXFA_Node* pNode, CXFA_CheckButton* button);
   ~CXFA_FFCheckButton() override;
 
   // CXFA_FFField
@@ -29,18 +33,20 @@ class CXFA_FFCheckButton : public CXFA_FFField {
   void OnProcessEvent(CFWL_Event* pEvent) override;
   void OnDrawWidget(CXFA_Graphics* pGraphics,
                     const CFX_Matrix& matrix) override;
+  FormFieldType GetFormFieldType() override;
 
   void SetFWLCheckState(XFA_CHECKSTATE eCheckState);
 
  private:
   bool CommitData() override;
   bool IsDataChanged() override;
-  void CapLeftRightPlacement(CXFA_Margin mgCap);
-  void AddUIMargin(int32_t iCapPlacement);
+  void CapLeftRightPlacement(const CXFA_Margin* captionMargin);
+  void AddUIMargin(XFA_AttributeEnum iCapPlacement);
   XFA_CHECKSTATE FWLState2XFAState();
 
-  IFWL_WidgetDelegate* m_pOldDelegate;
+  UnownedPtr<IFWL_WidgetDelegate> m_pOldDelegate;
   CFX_RectF m_rtCheckBox;
+  UnownedPtr<CXFA_CheckButton> button_;
 };
 
 #endif  // XFA_FXFA_CXFA_FFCHECKBUTTON_H_
