@@ -9,22 +9,22 @@
 
 #include <vector>
 
-#include "xfa/fxfa/parser/cxfa_nodelist.h"
+#include "xfa/fxfa/parser/cxfa_treelist.h"
 
 class CXFA_Document;
 class CXFA_Node;
 
-class CXFA_ArrayNodeList : public CXFA_NodeList {
+class CXFA_ArrayNodeList final : public CXFA_TreeList {
  public:
   explicit CXFA_ArrayNodeList(CXFA_Document* pDocument);
   ~CXFA_ArrayNodeList() override;
 
-  // From CXFA_NodeList.
-  int32_t GetLength() override;
-  bool Append(CXFA_Node* pNode) override;
-  bool Insert(CXFA_Node* pNewNode, CXFA_Node* pBeforeNode) override;
-  bool Remove(CXFA_Node* pNode) override;
-  CXFA_Node* Item(int32_t iIndex) override;
+  // From CXFA_TreeList.
+  size_t GetLength() override;
+  void Append(CXFA_Node* pNode) override;
+  void Insert(CXFA_Node* pNewNode, CXFA_Node* pBeforeNode) override;
+  void Remove(CXFA_Node* pNode) override;
+  CXFA_Node* Item(size_t iIndex) override;
 
   void SetArrayNodeList(const std::vector<CXFA_Node*>& srcArray);
 
