@@ -9,9 +9,10 @@
 
 #include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/fx_system.h"
-#include "xfa/fxfa/parser/cxfa_widgetdata.h"
+#include "core/fxcrt/unowned_ptr.h"
+#include "xfa/fxfa/parser/cxfa_node.h"
 
-class IFX_Locale;
+class LocaleIface;
 class CFX_DateTime;
 class CXFA_LocaleMgr;
 
@@ -28,7 +29,6 @@ class CXFA_LocaleMgr;
 class CXFA_LocaleValue {
  public:
   CXFA_LocaleValue();
-  CXFA_LocaleValue(const CXFA_LocaleValue& value);
   CXFA_LocaleValue(uint32_t dwType, CXFA_LocaleMgr* pLocaleMgr);
   CXFA_LocaleValue(uint32_t dwType,
                    const WideString& wsValue,
@@ -36,26 +36,29 @@ class CXFA_LocaleValue {
   CXFA_LocaleValue(uint32_t dwType,
                    const WideString& wsValue,
                    const WideString& wsFormat,
-                   IFX_Locale* pLocale,
+                   LocaleIface* pLocale,
                    CXFA_LocaleMgr* pLocaleMgr);
+  CXFA_LocaleValue(const CXFA_LocaleValue& that);
   ~CXFA_LocaleValue();
-  CXFA_LocaleValue& operator=(const CXFA_LocaleValue& value);
+
+  CXFA_LocaleValue& operator=(const CXFA_LocaleValue& that);
 
   bool ValidateValue(const WideString& wsValue,
                      const WideString& wsPattern,
-                     IFX_Locale* pLocale,
+                     LocaleIface* pLocale,
                      WideString* pMatchFormat);
 
   bool FormatPatterns(WideString& wsResult,
                       const WideString& wsFormat,
-                      IFX_Locale* pLocale,
+                      LocaleIface* pLocale,
                       XFA_VALUEPICTURE eValueType) const;
 
   void GetNumericFormat(WideString& wsFormat, int32_t nIntLen, int32_t nDecLen);
   bool ValidateNumericTemp(const WideString& wsNumeric,
                            const WideString& wsFormat,
-                           IFX_Locale* pLocale);
+                           LocaleIface* pLocale);
 
+  bool IsValid() const { return m_bValid; }
   WideString GetValue() const { return m_wsValue; }
   uint32_t GetType() const { return m_dwType; }
   double GetDoubleNum() const;
@@ -63,12 +66,10 @@ class CXFA_LocaleValue {
   CFX_DateTime GetDate() const;
   CFX_DateTime GetTime() const;
 
-  bool IsValid() const { return m_bValid; }
-
  private:
   bool FormatSinglePattern(WideString& wsResult,
                            const WideString& wsFormat,
-                           IFX_Locale* pLocale,
+                           LocaleIface* pLocale,
                            XFA_VALUEPICTURE eValueType) const;
   bool ValidateCanonicalValue(const WideString& wsValue, uint32_t dwVType);
   bool ValidateCanonicalDate(const WideString& wsDate, CFX_DateTime* unDate);
@@ -79,12 +80,12 @@ class CXFA_LocaleValue {
 
   bool ParsePatternValue(const WideString& wsValue,
                          const WideString& wsPattern,
-                         IFX_Locale* pLocale);
+                         LocaleIface* pLocale);
 
-  CXFA_LocaleMgr* m_pLocaleMgr;
+  UnownedPtr<CXFA_LocaleMgr> m_pLocaleMgr;
   WideString m_wsValue;
-  uint32_t m_dwType;
-  bool m_bValid;
+  uint32_t m_dwType = XFA_VT_NULL;
+  bool m_bValid = true;
 };
 
 #endif  // XFA_FXFA_PARSER_CXFA_LOCALEVALUE_H_
